/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.fleet.nameTable;

import com.sunlabs.archsim.fleet.nameTable.NameToAddressMap;

/**
 * Name table for sources and destinations.
 * A source or a destination and its address can be added to the table.
 * Note that a same name could be used on bothe the source side and the
 * destination side, with different addresses.
 * 
 * @author ib27688
 *
 */
public class SourceDestinationNameTable {
	protected static NameToAddressMap sourceMap= new NameToAddressMap();
	protected static NameToAddressMap destinationMap= new NameToAddressMap();
	
	protected static SourceDestinationNameTable myInstance= null;
	
	// a singleton
	protected SourceDestinationNameTable() {
	}
	
	
	// not thread safe
	public static SourceDestinationNameTable getInstance() {
		if (myInstance==null) myInstance= new SourceDestinationNameTable();
		return myInstance;
	}
	
	
	/**
	 * Get teh address for a destination.
	 * @param name destination name
	 * @return destination address. -1 if unknown
	 */
	public int getSourceAddress(String name) throws NameUnknownException {
		return sourceMap.getAddress(name);
	} // getSourceAddress
	
	
	public int getDestinationAddress(String name) throws NameUnknownException {
		return destinationMap.getAddress(name);
	}
	
	/**
	 * Add a source and its address. Return the old address if
	 * the same name already existed.
	 * @param name source name
	 * @param address source address
	 * @return null if the name is new, old address otherwise
	 */

	public Integer addSource(String name, int address) {
		return sourceMap.addName(name, address);
	}

	/**
	 * Add a destination and its address. Return the old address if
	 * the same name already existed.
	 * @param name destination name
	 * @param address destination address
	 * @return null if the name is new, old address otherwise
	 */
	public Integer addDestination(String name, int address) {
		return destinationMap.addName(name, address);
	}
	
	/**
	 * Clear both maps - for sources and destinations.
	 */
	public void clear() {
		sourceMap.clear();
		destinationMap.clear();
	}
	
	public String toString() {
		String s = "Sources:" 
			+ sourceMap.toString()
			+ "\nDestinations:"
			+ destinationMap.toString();
		
		return s;
	} // toString
	
} // SourceDestinationNameTable
